@@ -388,9 +388,9 @@ public:
                 } else {
                     sub = format_type(type_name, v.m_name, use_ref, init_default ? "\"\"" : "");
                 }
-            } else if (ASR::is_a<ASR::Struct_t>(*v_m_type)) {
+            } else if (ASR::is_a<ASR::StructType_t>(*v_m_type)) {
                 // TODO: handle this
-                ASR::Struct_t* t = ASR::down_cast<ASR::Struct_t>(v_m_type);
+                ASR::StructType_t* t = ASR::down_cast<ASR::StructType_t>(v_m_type);
                 std::string der_type_name = ASRUtils::symbol_name(t->m_derived_type);
                 if (is_array) {
                     generate_array_decl(sub,
@@ -978,8 +978,8 @@ public:
                 std::string type_name = "String";
                 generate_array_decl(
                     out, std::string(v->m_name), type_name, _dims, nullptr, n_dims, true, true);
-            } else if (ASR::is_a<ASR::Struct_t>(*v->m_type)) {
-                ASR::Struct_t* t = ASR::down_cast<ASR::Struct_t>(v->m_type);
+            } else if (ASR::is_a<ASR::StructType_t>(*v->m_type)) {
+                ASR::StructType_t* t = ASR::down_cast<ASR::StructType_t>(v->m_type);
                 std::string der_type_name = ASRUtils::symbol_name(t->m_derived_type);
                 generate_array_decl(
                     out, std::string(v->m_name), der_type_name, _dims, nullptr, n_dims, true, true);
@@ -1435,10 +1435,9 @@ public:
     {
         std::string indent(indentation_level * indentation_spaces, ' ');
         std::string out = "[";
-        for (size_t i = 0; i < x.n_args; i++) {
-            visit_expr(*x.m_args[i]);
-            out += src;
-            if (i < x.n_args - 1)
+        for (size_t i = 0; i < (size_t) ASRUtils::get_fixed_size_of_array(x.m_type); i++) {
+            out += ASRUtils::fetch_ArrayConstant_value(x, i);
+            if (i < (size_t) ASRUtils::get_fixed_size_of_array(x.m_type) - 1)
                 out += ", ";
         }
         out += "]";
@@ -1901,7 +1900,6 @@ public:
             SET_INTRINSIC_NAME(Expm1, "expm1");
             SET_INTRINSIC_NAME(Trunc, "trunc");
             SET_INTRINSIC_NAME(Fix, "fix");
-            SET_INTRINSIC_NAME(Kind, "kind");
             SET_INTRINSIC_NAME(StringContainsSet, "verify");
             SET_INTRINSIC_NAME(StringFindSet, "scan");
             SET_INTRINSIC_NAME(SubstrIndex, "index");
