@@ -376,9 +376,9 @@ namespace IntrinsicElementalFunctionRegistry {
         {static_cast<int64_t>(IntrinsicElementalFunctions::SubstrIndex),
             {&SubstrIndex::instantiate_SubstrIndex, &SubstrIndex::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::MinExponent),
-            {&MinExponent::instantiate_MinExponent, &MinExponent::verify_args}},
+            {nullptr, &MinExponent::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::MaxExponent),
-            {&MaxExponent::instantiate_MaxExponent, &MaxExponent::verify_args}},
+            {nullptr, &MaxExponent::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Abs),
             {&Abs::instantiate_Abs, &Abs::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Aimag),
